"""
Qt GUI for Thorlabs PM100D power meter.

Copyright (c) 2022, Jacob Feder, Ben Soloway
All rights reserved.
"""
import logging

from pyqtgraph.Qt import QtWidgets
from pyqtgraph import SpinBox

logger = logging.getLogger(__name__)

class PM100Widget(QtWidgets.QWidget):
    """Qt widget for controlling the PM100D power meter."""

    def __init__(self, pm100d_driver):
        """
        Args:
            pm100d_driver: PM100D driver.
        """
        super().__init__()

        self.pm = pm100d_driver

        # top level layout
        layout = QtWidgets.QGridLayout()
        layout_row = 0

        # power spinbox
        layout.addWidget(QtWidgets.QLabel('Correction wavelength'), layout_row, 0)
        self.corr_wave_spinbox = SpinBox(value=488, suffix='nm', siPrefix=True, bounds=(400, 1100), dec=True, minStep=5)
        self.corr_wave_spinbox.sigValueChanged.connect(lambda spinbox: self.pm.set_correction_wavelength(spinbox.value()))
        self.corr_wave_spinbox.sigValueChanged.connect(self.update_corr_wave)
        self.corr_wave_spinbox.setValue(value=488)
        layout.addWidget(self.corr_wave_spinbox, layout_row, 1)
        layout_row += 1

        # correction wavelength label
        self.corr_wave_label = QtWidgets.QLabel('')
        self.update_corr_wave()
        layout.addWidget(self.corr_wave_label, layout_row, 1)
        # get state button
        self.get_corr_wave_button = QtWidgets.QPushButton('Get Correction Wavelength')
        self.get_corr_wave_button.clicked.connect(self.update_corr_wave)
        layout.addWidget(self.get_corr_wave_button, layout_row, 0)
        layout_row += 1

        # power label
        self.power_label = QtWidgets.QLabel('')
        self.update_power()
        layout.addWidget(self.power_label, layout_row, 1)
        # get power button
        self.get_power_button = QtWidgets.QPushButton('Get power')
        self.get_power_button.clicked.connect(self.update_power)
        layout.addWidget(self.get_power_button, layout_row, 0)
        layout_row += 1

        # take up any additional space in the final column with padding
        layout.setColumnStretch(2, 1)
        # take up any additional space in the final row with padding
        layout.setRowStretch(layout_row, 1)

        self.setLayout(layout)

    def update_power(self):
        """Query the laser for the current state then update the state text box."""
        self.power_label.setText(str(self.pm.power()))

    def update_corr_wave(self):
        self.corr_wave_label.setText(str(self.pm.get_correction_wavelength()))



# pip install Adafruit_BBIO
import Adafruit_BBIO.GPIO as GPIO
from flask import Flask
from markupsafe import escape
import os

app = Flask(__name__)

@app.route('/')
def hello():
    return '<p>Example of terminal is P8_10. To output high use /gpio/output/high/terminal. To output low use /gpio/output/low/terminal</p>'

@app.route('/restart')
def restart():
    GPIO.cleanup()
    os._exit(0)

@app.route('/gpio/output/high/<string:terminal>')
def output_high(terminal):
    GPIO.setup(terminal, GPIO.OUT)
    GPIO.output(terminal, GPIO.HIGH)
    return f'{escape(terminal)} is high'

@app.route('/gpio/output/low/<string:terminal>')
def output_low(terminal):
    GPIO.setup(terminal, GPIO.OUT)
    GPIO.output(terminal, GPIO.LOW)
    return f'{escape(terminal)} is low'

if __name__ == '__main__':
    app.run(host='0.0.0.0')



"""Driver for the Thorlabs PM100D.

Copyright (c) 2022, Jacob Feder, Ben Soloway
All rights reserved.
"""
import logging

from pyvisa import ResourceManager

logger = logging.getLogger(__name__)

class PM100D:
    def __init__(self, address):
        """
        Args:
            address: PyVISA resource path.
        """
        self.rm = ResourceManager('@py')
        self.address = address

        logger.info(f'Connected to PM100D [{self.address}].')

    def __enter__(self):
        return self

    def __exit__(self, *args):
        self.close()

    def __str__(self):
        return f'{self.address} {self.idn}'

    def open(self):
        try:
            self.device = self.rm.open_resource(self.address)
        except Exception as err:
            raise ConnectionError(f'Failed connecting to PM100D @ [{self.address}]') from err
        # 1 second timeout
        self.device.timeout = 1000
        self.idn = self.device.query('*IDN?')
        logger.info(f'Connected to PM100D [{self}].')
        return self

    def close(self):
        self.pm.close()

    def idn(self):
        return self.pm.query('*IDN?')

    def power(self):
        return float(self.pm.query('MEAS:POWER?'))

    def get_correction_wavelength(self):
        return float(self.pm.query('SENS:CORR:WAV?'))

    def set_correction_wavelength(self, wavelength):
        self.pm.write('SENSE:CORRECTION:WAVELENGTH {}'.format(wavelength))

    def correction_wavelength_range(self):
        cmd = 'SENSE:CORRECTION:WAVELENGTH? {}'
        cmd_vals = ['MIN', 'MAX']
        return tuple(float(self.pm.query(cmd.format(cmd_val))) for cmd_val in cmd_vals)
